package com.baidu.cloud.videoplayer.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.SurfaceTexture;
import android.opengl.GLES30;
import android.opengl.GLSurfaceView;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Surface;
import android.view.View;

import com.baidu.cloud.media.player.IMediaPlayer;
import com.baidu.cloud.media.player.render.record.IRecordListener;
import com.baidu.cloud.videoplayer.demo.helper.HdrKitWrapper;

import java.lang.ref.WeakReference;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

public class GLSurfaceRenderView extends GLSurfaceView implements IRenderView {

    private static final String TAG = "GLSurfaceRenderView";
    private MeasureHelper mMeasureHelper;
    private GLVideoRender mVideoRender;
    private boolean bDoHDR;
    private String mAppid;
    private int mVideoWidth;
    private int mVideoHeight;

    private long mRenderFrameNum = 0;
    private long mRenderFrameTimeMs = 0;
    private float mAvgRenderFps = 0f;

//    private EContextFactory mContextFactory;

    public GLSurfaceRenderView(Context context) {
        super(context);
        initView(context);
    }

    public GLSurfaceRenderView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initView(context);
    }

    private void initView(Context context) {
        mMeasureHelper = new MeasureHelper(this);
        setEGLContextClientVersion(3);
        mVideoRender = new GLVideoRender(context, this);
        setRenderer(mVideoRender);
    }

    @Override
    public View getView() {
        return this;
    }

    @Override
    public boolean shouldWaitForResize() {
        return false;
    }

    @Override
    public void setVideoSize(int videoWidth, int videoHeight) {
        if (videoWidth > 0 && videoHeight > 0) {
            mVideoWidth = videoWidth;
            mVideoHeight = videoHeight;
            mMeasureHelper.setVideoSize(videoWidth, videoHeight);
            requestLayout();
        }
    }

    @Override
    public void setVideoSampleAspectRatio(int videoSarNum, int videoSarDen) {
        if (videoSarNum > 0 && videoSarDen > 0) {
            mMeasureHelper.setVideoSampleAspectRatio(videoSarNum, videoSarDen);
            requestLayout();
        }
    }

    @Override
    public void setVideoRotation(int degree) {
        mMeasureHelper.setVideoRotation(degree);
        setRotation(degree);
    }

    @Override
    public void setAspectRatio(int aspectRatio) {
        mMeasureHelper.setAspectRatio(aspectRatio);
        requestLayout();
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        mMeasureHelper.doMeasure(widthMeasureSpec, heightMeasureSpec);
        setMeasuredDimension(mMeasureHelper.getMeasuredWidth(), mMeasureHelper.getMeasuredHeight());
    }

    @Override
    public void addRenderCallback(IRenderCallback callback) {
        mVideoRender.addRenderCallback(callback);
    }

    @Override
    public void removeRenderCallback(IRenderCallback callback) {
        mVideoRender.removeRenderCallback(callback);
    }

    @Override
    public void release() {
        mVideoRender.release();
    }

    @Override
    public void destroy() {

    }

    @Override
    public Bitmap getBitmap() {
        return mVideoRender.getBitmap();
    }

    @Override
    public void startRecord(String filePath) {
        // not support
    }

    @Override
    public void setRecordListener(IRecordListener listener) {
        // not support
    }

    @Override
    public void stopRecord() {
        // not support
    }

    @Override
    public void onUpdateAudioData(byte[] bytes, int byteSize) {
        // not support
    }

    private int mMediaPlayerCode = 0;

    public void setCurrentMediaPlayerCode(int currentCode) {
        mMediaPlayerCode = currentCode;
    }

    public int getCurrentMediaPlayerCode() {
        return mMediaPlayerCode;
    }

    private SurfaceTexture lastSurfaceTexture;

    public SurfaceTexture getLastSurfaceTexture() {
        return lastSurfaceTexture;
    }

    public void setLastSurfaceTexture(SurfaceTexture surfaceTexture) {
        lastSurfaceTexture = surfaceTexture;
    }

    public IRenderView.ISurfaceHolder getSurfaceHolder() {
        return new InternalSurfaceHolder(this);
    }

    private static final class InternalSurfaceHolder implements IRenderView.ISurfaceHolder {
        private GLSurfaceRenderView mGLSurfaceRenderView;

        public InternalSurfaceHolder(GLSurfaceRenderView glSurfaceRenderView) {
            mGLSurfaceRenderView = glSurfaceRenderView;
        }

        public void bindToMediaPlayer(IMediaPlayer mp) {
            if (mp == null || mGLSurfaceRenderView.mVideoRender.getSurfaceTexture() == null) {
                // if reSetRender is invoked twice during one playing, surfaceTexture maybe null
                return;
            }
            // Tricky: 由 Object 类定义的 hashCode 方法确实会针对不同的对象返回不同的整数。
            // （这一般是通过将该对象的内部地址转换成一个整数来实现的，但是 Java语言不需要这种实现技巧。）
            if (mp.hashCode() != mGLSurfaceRenderView.getCurrentMediaPlayerCode()) {
                // new player
                mp.setSurface(openSurface());
            } else {
                // old player
                if (!mGLSurfaceRenderView.getLastSurfaceTexture()
                        .equals(mGLSurfaceRenderView.mVideoRender.getSurfaceTexture())) {
                    mGLSurfaceRenderView.mVideoRender.
                            setSurfaceTexture(mGLSurfaceRenderView.getLastSurfaceTexture());
                }
            }
            mGLSurfaceRenderView.setCurrentMediaPlayerCode(mp.hashCode());
        }

        @Override
        public IRenderView getRenderView() {
            return mGLSurfaceRenderView;
        }

        @Override
        public Surface openSurface() {
            return new Surface(mGLSurfaceRenderView.mVideoRender.getSurfaceTexture());
        }
    }

    public float getRenderFps() {
        return mAvgRenderFps;
    }

    public void setDoHDR(boolean doHDR, String appid) {
        bDoHDR = doHDR;
        mAppid = appid;
        if (doHDR && mVideoRender.mHdrKit == null) {
            mVideoRender.mHdrKit = HdrKitWrapper.constructHdrKit();
            HdrKitWrapper.init(mVideoRender.mHdrKit, getContext(), mAppid);
        }
    }

    public void setHdrStaticMetadata(int metadataType, byte[] metadata) {
        if (mVideoRender.mHdrKit != null) {
            HdrKitWrapper.setHdrStaticMetadata(mVideoRender.mHdrKit, metadataType, metadata);
        }
    }

    public void setVividMetadata(byte[] metadata) {
        if (mVideoRender.mHdrKit != null) {
            HdrKitWrapper.setVividMetadata(mVideoRender.mHdrKit, metadata);
        }
    }

    public void setHdrContentType(HdrKitWrapper.ContentType contentType) {
        if (mVideoRender.mHdrKit != null) {
            HdrKitWrapper.setInputVideoContentType(mVideoRender.mHdrKit, contentType);
        }
    }

//    public void setDoVideoEnhance(boolean doVideoEnhance, String appid) {
//        bDoVideoEnhance = doVideoEnhance;
//        mAppid = appid;
//        if (doVideoEnhance && mVideoRender.mVideoEnhanceFilter == null) {
//            mVideoRender.mVideoEnhanceFilter = new VideoEnhanceFilter();
//            mVideoRender.mVideoEnhanceFilter.setup(getContext(), mContextFactory.getContext());
//        }
//    }

    private static class GLVideoRender
            implements GLSurfaceView.Renderer, SurfaceTexture.OnFrameAvailableListener {

        private GLTextureRender mGLTextureRender;
        private SurfaceTexture mSurfaceTexture;
        private boolean updateSurface = false;
        private int mFboId = -1;
        private int mInputTex = -1;
        private int mOutputTex = -1;
        private Object mHdrKit;

        private WeakReference<GLSurfaceRenderView> mWeakRenderView;
        private Map<IRenderCallback, Object> mRenderCallbackMap = new ConcurrentHashMap<>();

        private int width;
        private int height;

        public GLVideoRender(Context context, GLSurfaceRenderView renderView) {
            mGLTextureRender = new GLTextureRender();
            mWeakRenderView = new WeakReference<>(renderView);
        }

        public SurfaceTexture getSurfaceTexture() {
            return mSurfaceTexture;
        }

        public void setSurfaceTexture(SurfaceTexture surfaceTexture) {
            if (surfaceTexture.equals(mSurfaceTexture)) {
                return;
            }
            if (mSurfaceTexture != null) {
                mSurfaceTexture.release();
            }
            mSurfaceTexture = surfaceTexture;
            mSurfaceTexture.setOnFrameAvailableListener(this);

            synchronized(this) {
                updateSurface = false;
            }
        }

        public void addRenderCallback(IRenderCallback callback) {
            mRenderCallbackMap.put(callback, callback);

            ISurfaceHolder surfaceHolder = null;
            if (mSurfaceTexture != null) {
                surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
                callback.onSurfaceCreated(surfaceHolder, width, height);
            }

            if (width > 0 && height > 0) {
                if (surfaceHolder == null) {
                    surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
                }
                callback.onSurfaceChanged(surfaceHolder, 0, width, height);
            }
        }

        public void removeRenderCallback(IRenderCallback callback) {
            mRenderCallbackMap.remove(callback);
        }

        public Bitmap getBitmap() {
            if (width > 0 && height > 0) {
                return mGLTextureRender.getBitmap(width, height);
            }
            return null;
        }

        @Override
        public void onDrawFrame(GL10 glUnused) {
            // 在单独的线程
            synchronized(this) {
                if (updateSurface) {
                    mSurfaceTexture.updateTexImage();
                    updateSurface = false;
                }
            }
            if (mWeakRenderView.get().bDoHDR) {
                synchronized (this) {
                    if (mHdrKit != null) {
                        GLTextureRender.bindTextureToFBO(mInputTex, GLES30.GL_TEXTURE_2D, mFboId);
                        mGLTextureRender.drawFrame(mSurfaceTexture, mFboId);

                        HdrKitWrapper.prepareWithInputTexId(mHdrKit, mInputTex);
                        HdrKitWrapper.setOutputTexId(mHdrKit, mOutputTex);
                        HdrKitWrapper.setInputSize(mHdrKit, width, height);
                        HdrKitWrapper.setOutputSize(mHdrKit, width, height);
                        HdrKitWrapper.drawFrameFromTexId(mHdrKit);

                        mGLTextureRender.drawFrame(mOutputTex, 0);
                    } else {
                        mGLTextureRender.drawFrame(mSurfaceTexture, 0);
                    }
                }
            } else {
                mGLTextureRender.drawFrame(mSurfaceTexture, 0);
            }
        }

        @Override
        public void onSurfaceChanged(GL10 glUnused, int width, int height) {
            if (mInputTex != -1) {
                GLTextureRender.destroyTextureObject(mInputTex);
            }
            mInputTex = GLTextureRender.createTextureObject(GLES30.GL_TEXTURE_2D, width, height);

            if (mOutputTex != -1) {
                GLTextureRender.destroyTextureObject(mOutputTex);
            }
            mOutputTex = GLTextureRender.createTextureObject(GLES30.GL_TEXTURE_2D, width, height);

            mGLTextureRender.surfaceChanged(width, height);
            this.width = width;
            this.height = height;

            ISurfaceHolder surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceChanged(surfaceHolder, 0, width, height);
            }
        }

        @Override
        public void onSurfaceCreated(GL10 gl, EGLConfig config) {
            mFboId = GLTextureRender.createFrameBufferObject();

            mGLTextureRender.surfaceCreated();
            /*
             * Create the SurfaceTexture that will feed this textureID,
             * and pass it to the MediaPlayer
             */
            mSurfaceTexture = new SurfaceTexture(mGLTextureRender.getTextureId());
            mSurfaceTexture.setOnFrameAvailableListener(this);

            if (mWeakRenderView.get() == null) {
                // should never come in
                Log.e(TAG, "!!!!!Too bad, textureview in callback is released. function will not work normally");
            } else {
                // only set once
                if (mWeakRenderView.get().getLastSurfaceTexture() == null) {
                    mWeakRenderView.get().setLastSurfaceTexture(mSurfaceTexture);
                }
            }

            width = 0;
            height = 0;

            ISurfaceHolder surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceCreated(surfaceHolder, 0, 0);
            }

            synchronized(this) {
                updateSurface = false;
            }
        }

        @Override
        public synchronized void onFrameAvailable(SurfaceTexture surface) {
            // 在主线程
            updateSurface = true;
        }

        public void release() {
            if (mSurfaceTexture != null) {
                mSurfaceTexture.setOnFrameAvailableListener(null);
                mSurfaceTexture.release();
            }
            synchronized (this) {
                if (mHdrKit != null) {
                    HdrKitWrapper.release(mHdrKit);
                    mHdrKit = null;
                }
            }
            width = 0;
            height = 0;

            ISurfaceHolder surfaceHolder = new InternalSurfaceHolder(mWeakRenderView.get());
            for (IRenderCallback renderCallback : mRenderCallbackMap.keySet()) {
                renderCallback.onSurfaceDestroyed(surfaceHolder);
            }
        }
    }

//    public class EContextFactory implements GLSurfaceView.EGLContextFactory {
//
//        private static final String TAG = "EContextFactory";
//
//        private final int EGL_CLIENT_VERSION = 3;
//
//        private static final int EGL_CONTEXT_CLIENT_VERSION = 0x3098;
//
//        private EGLContext mContext;
//
//        public EGLContext getContext() {
//            return mContext;
//        }
//
//        @Override
//        public EGLContext createContext(final EGL10 egl, final EGLDisplay display, final EGLConfig config) {
//            final int[] attrib_list;
//            attrib_list = new int[]{EGL_CONTEXT_CLIENT_VERSION, EGL_CLIENT_VERSION, EGL_NONE};
//            mContext =  egl.eglCreateContext(display, config, EGL_NO_CONTEXT, attrib_list);
//            return mContext;
//        }
//
//        @Override
//        public void destroyContext(final EGL10 egl, final EGLDisplay display, final EGLContext context) {
//            if (!egl.eglDestroyContext(display, context)) {
//                Log.e(TAG, "display:" + display + " context: " + context);
//                throw new RuntimeException("eglDestroyContex" + egl.eglGetError());
//            }
//        }
//
//    }
}
