package com.baidu.cloud.videoplayer.demo.helper;

import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class HttpDnsUtil {
    private static final String TAG = "HttpDnsUtil";
    private static final String DNS_SERVER = "http://httpdns.bcelive.com/";
    private static OkHttpClient mOkHttpClient = new OkHttpClient();
    private static ScheduledExecutorService executorService = Executors.newSingleThreadScheduledExecutor();
    private static final int DEFAULT_TTL = 300;

    public static class IpInfo {
        public String domainName;
        public ArrayList<String> ipAddrs;
        public int ttlMs;

        IpInfo() {
            ipAddrs = new ArrayList<>();
            ttlMs = DEFAULT_TTL;
        }
    }

    public static void getIpFromDomain(final String domain, final IDnsCallback callback) {
        executorService.execute(new Runnable() {
            @Override
            public void run() {
                Log.d(TAG, "try to get Ip from domain " + domain);
                String requestUrl = DNS_SERVER + "?dns=" + domain;
                Request request = new Request.Builder()
                        .url(requestUrl)
                        .get()
                        .build();
                Call call = mOkHttpClient.newCall(request);
                call.enqueue(new Callback() {
                    @Override
                    public void onFailure(Call call, IOException e) {
                        if (callback != null) {
                            callback.onFail();
                        }
                    }

                    @Override
                    public void onResponse(Call call, Response response) throws IOException {
                        if (callback != null && response.body() != null) {
                            callback.onSuccess(parseDnsJson(response.body().string(), domain));
                        }
                    }
                });
            }
        });
    }

    private static IpInfo parseDnsJson(String response, String domain) {
        IpInfo ipInfo = new IpInfo();
        try {
            JSONObject jsonObject = new JSONObject(response);
            JSONObject dataObject = jsonObject.getJSONObject("data").getJSONObject(domain);
            JSONArray ipArray = dataObject.getJSONArray("ips");
            for (int i = 0; i < ipArray.length(); i++) {
                ipInfo.ipAddrs.add(ipArray.getString(i));
            }
            ipInfo.ttlMs = dataObject.getInt("ttl");
            ipInfo.domainName = domain;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return ipInfo;
    }

    public interface IDnsCallback {
        void onSuccess(IpInfo ipInfo);
        void onFail();
    }
}
