package com.baidu.cloud.videoplayer.demo.rtc;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.R;

public class RTCPlayerLoginActivity extends AppCompatActivity {
    private static final String TAG = "RTCPlayerLoginActivity";

    private static final int REQUEST_STORAGE_WRITE_PERMISSION_ID = 102;

    private static final String DEFAULT_STREAM_URL =
            "webrtc://test-pl-central.bigenemy.cn/myapp/playersdkdemo.flv";
    private static final String DEFAULT_SIGNAL_URL =
            "http://test-pl-central.bigenemy.cn/brtc/v3/pullstream";

    // UI references.
    private EditText mStreamView;
    private EditText mSignalView;
    private CheckBox mLssHttpPathCb;
    private CheckBox mLssUdpPathCb;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if ((getIntent().getFlags() & Intent.FLAG_ACTIVITY_BROUGHT_TO_FRONT) != 0) {
            finish();
            return;
        }
        setContentView(R.layout.activity_rtcplayer_login);

        populateAutoComplete();

        mLssHttpPathCb = findViewById(R.id.checkbox_lss);
        mLssUdpPathCb = findViewById(R.id.checkbox_lss_udp);


        mLssHttpPathCb.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    mLssUdpPathCb.setChecked(false);
                }
            }
        });
        mLssUdpPathCb.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    mLssHttpPathCb.setChecked(false);
                }
            }
        });

        mStreamView = findViewById(R.id.roomid);
        try {
            mStreamView.setText(PreferenceManager.getDefaultSharedPreferences(this)
                    .getString(C.KEY_RTC_STREAMURL, DEFAULT_STREAM_URL));
        } catch (Exception e) {
            e.printStackTrace();
            mStreamView.setText("");
        }

        mStreamView.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int id, KeyEvent keyEvent) {
                if (id == EditorInfo.IME_ACTION_DONE || id == EditorInfo.IME_NULL) {

                    if (mStreamView.getText().toString().isEmpty()) {
                        return true;
                    }
                    PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).edit()
                            .putString(C.KEY_RTC_STREAMURL, mStreamView.getText().toString()).apply();
                    return true;
                }
                return false;
            }
        });

        mSignalView = findViewById(R.id.signalurl);
        try {
            mSignalView.setText(PreferenceManager.getDefaultSharedPreferences(this)
                    .getString(C.KEY_RTC_SIGNALURL, DEFAULT_SIGNAL_URL));
        } catch (Exception e) {
            e.printStackTrace();
            mSignalView.setText("");
        }

        mSignalView.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int id, KeyEvent keyEvent) {
                if (id == EditorInfo.IME_ACTION_DONE || id == EditorInfo.IME_NULL) {

                    if (mSignalView.getText().toString().isEmpty()) {
                        return true;
                    }
                    PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).edit()
                            .putString(C.KEY_RTC_SIGNALURL, mSignalView.getText().toString()).apply();
                    return true;
                }
                return false;
            }
        });

        Button mSignInButton = (Button) findViewById(R.id.sign_in_button);
        mSignInButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                if (mStreamView.getText().toString().isEmpty() || mSignalView.getText().toString().isEmpty()) {
                    return;
                }
                PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).edit()
                        .putString(C.KEY_RTC_STREAMURL, mStreamView.getText().toString()).apply();
                PreferenceManager.getDefaultSharedPreferences(getApplicationContext()).edit()
                        .putString(C.KEY_RTC_SIGNALURL, mSignalView.getText().toString()).apply();

                Intent intent = new Intent(RTCPlayerLoginActivity.this, RTCPlayerPagerActivity.class);
                intent.putExtra(C.KEY_RTC_STREAMURL, mStreamView.getText().toString());
                intent.putExtra(C.KEY_RTC_SIGNALURL, mSignalView.getText().toString());
                if (mLssHttpPathCb.isChecked()) {
                    intent.putExtra(C.KEY_RTC_SIGNAL_TYPE, C.RTC_SIGNAL_TYPE_HTTP);
                } else {
                    intent.putExtra(C.KEY_RTC_SIGNAL_TYPE, C.RTC_SIGNAL_TYPE_UDP);
                }
                startActivity(intent);


            }
        });
    }

    private void populateAutoComplete() {
        if (!mayRequestPermissions()) {
            return;
        }
    }

    private boolean mayRequestPermissions() {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            return true;
        }

        requestPermissions();

        if (checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)
                == PackageManager.PERMISSION_GRANTED) {
            return false;
        }
        return false;
    }

    /**
     * Callback received when a permissions request has been completed.
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        if (requestCode == REQUEST_STORAGE_WRITE_PERMISSION_ID) {
            if (grantResults.length == 1 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                populateAutoComplete();
            }
        }
    }

    private void requestPermissions() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (this.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)
                    != PackageManager.PERMISSION_GRANTED) {
                this.requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
                        REQUEST_STORAGE_WRITE_PERMISSION_ID);
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
    }
}
